@@ -12,7 +12,7 @@ from redis.exceptions import ResponseError
 import redis.client
 
 
-__version__ = '0.4.2'
+__version__ = '0.4.3'
 DATABASES = {}
 
 _libc = CDLL(find_library('c'))
@@ -38,7 +38,7 @@ class _StrKeyDict(MutableMapping):
         self._dict[str(key)] = value
 
     def __delitem__(self, key):
-        del self._dict[key]
+        del self._dict[str(key)]
 
     def __len__(self):
         return len(self._dict)
@@ -320,6 +320,12 @@ class FakeStrictRedis(object):
     getrange = substr
 
     def ttl(self, name):
+        return self._ttl(name)
+
+    def pttl(self, name):
+        return self._ttl(name, 1000)
+
+    def _ttl(self, name, multiplier=1):
         if name not in self._db:
             return None
 
@@ -331,9 +337,9 @@ class FakeStrictRedis(object):
         if now > exp_time:
             return None
         else:
-            return round((exp_time - now).days * 3600 * 24
-                         + (exp_time - now).seconds
-                         + (exp_time - now).microseconds / 1E6)
+            return round(((exp_time - now).days * 3600 * 24
+                          + (exp_time - now).seconds
+                          + (exp_time - now).microseconds / 1E6) * multiplier)
 
     def type(self, name):
         pass
@@ -349,6 +355,8 @@ class FakeStrictRedis(object):
         for name in names:
             try:
                 del self._db[name]
+                if name in self._db._ex_keys:
+                    del self._db._ex_keys[name]
                 deleted += 1
             except KeyError:
                 continue
@@ -380,7 +388,7 @@ class FakeStrictRedis(object):
             raise redis.RedisError(
                 "RedisError: ``start`` and ``num`` must both be specified")
         try:
-            data = self._db[name][:]
+            data = list(self._db[name])[:]
             if by is not None:
                 # _sort_using_by_arg mutates data so we don't
                 # need need a return value.
@@ -389,10 +397,10 @@ class FakeStrictRedis(object):
                 data.sort(key=self._strtod_key_func)
             else:
                 data.sort()
-            if not (start is None and num is None):
-                data = data[start:start + num]
             if desc:
                 data = list(reversed(data))
+            if not (start is None and num is None):
+                data = data[start:start + num]
             if store is not None:
                 self._db[store] = data
                 return len(data)
@@ -647,6 +655,8 @@ class FakeStrictRedis(object):
         """
         if not mapping:
             raise redis.DataError("'hmset' with 'mapping' of length 0")
+        for k, v in mapping.items():
+          mapping[k] = str(v)
         self._db.setdefault(name, _StrKeyDict()).update(mapping)
         return True
 
@@ -664,7 +674,7 @@ class FakeStrictRedis(object):
         "Add ``value`` to set ``name``"
         a_set = self._db.setdefault(name, set())
         card = len(a_set)
-        a_set |= set(values)
+        a_set |= set(map(str, values))
         return len(a_set) - card
 
     def scard(self, name):
@@ -759,6 +769,11 @@ class FakeStrictRedis(object):
         self._db[dest] = union
         return len(union)
 
+    def _get_zelement_range(self, min, max):
+        # This will also handle the case when
+        # min/max are '-inf', '+inf'
+        return float(min), float(max)
+
     def zadd(self, name, *args, **kwargs):
         """
         Set any number of score, element-name pairs to the key ``name``. Pairs
@@ -797,6 +812,7 @@ class FakeStrictRedis(object):
 
     def zcount(self, name, min, max):
         found = 0
+        min, max = self._get_zelement_range(min, max)
         for score in self._db.get(name, {}).values():
             if min <= score <= max:
                 found += 1
@@ -857,7 +873,7 @@ class FakeStrictRedis(object):
             return [(k, all_items[k]) for k in items]
 
     def _get_zelements_in_order(self, all_items, reverse=False):
-        by_keyname = sorted(all_items.items(), key=lambda x: x[0])
+        by_keyname = sorted(all_items.items(), key=lambda x: x[0], reverse=reverse)
         in_order = sorted(by_keyname, key=lambda x: x[1], reverse=reverse)
         return [el[0] for el in in_order]
 
@@ -883,6 +899,7 @@ class FakeStrictRedis(object):
                                    "be specified")
         all_items = self._db.get(name, {})
         in_order = self._get_zelements_in_order(all_items, reverse=reverse)
+        min, max = self._get_zelement_range(min, max)
         matches = []
         for item in in_order:
             if min <= all_items[item] <= max:
@@ -940,6 +957,7 @@ class FakeStrictRedis(object):
         between ``min`` and ``max``. Returns the number of elements removed.
         """
         all_items = self._db.get(name, {})
+        min, max = self._get_zelement_range(min, max)
         removed = 0
         for key in all_items.copy():
             if min <= all_items[key] <= max:
